@@ -1,4 +1,4 @@
-//! `SplittibleServerCodec` and `SplittableClientCodec` are defined in this module, and they are implemented
+//! `SplittibleCodec` is defined in this module, and they are implemented
 //! for the `DefaultCodec`
 //! Default codec implementations are feature gated behind the following features
 //! `serde_bincode`, `serde_json`, `serde_cbor`, `serde_rmp`.
@@ -6,20 +6,14 @@
 use async_trait::async_trait;
 use cfg_if::cfg_if;
 use erased_serde as erased;
-use futures::stream::{SplitSink, SplitStream};
-use futures::{Sink, Stream};
 use std::marker::PhantomData;
-use tungstenite::Message as WsMessage;
 
 use crate::error::Error;
-use crate::message::{MessageId, Metadata, RequestHeader, ResponseHeader};
-use crate::transport::ws::{CanSink, SinkHalf, StreamHalf, WebSocketConn};
-// use crate::util::GracefulShutdown;
+use crate::message::{MessageId, Metadata};
+use crate::protocol::InboundBody;
 
 pub mod split;
 
-pub(crate) type RequestDeserializer = Box<dyn erased::Deserializer<'static> + Send + 'static>;
-
 cfg_if! {
     if #[cfg(feature = "http_tide")] {
         use tide_websockets as tide_ws;
@@ -167,13 +161,17 @@ cfg_if! {
     }
 }
 
+/// Type state for AsyncRead and AsyncWrite connections (ie. raw TCP)
 #[cfg(any(feature = "async_std_runtime", feature = "tokio_runtime"))]
-/// type state for AsyncRead and AsyncWrite connections (ie. raw TCP)
 pub(crate) struct ConnTypeReadWrite {}
 
-/// type state for PayloadRead and PayloadWrite connections (ie. WebSocket)
+/// Type state for PayloadRead and PayloadWrite connections (ie. WebSocket)
+#[cfg(any(feature = "ws_tokio", feature = "ws_async_std"))]
 pub(crate) struct ConnTypePayload {}
 
+/// Reserved type state for Reader/Writer for Codec
+pub struct Reserved {}
+
 /// Default codec. `Codec` is re-exported as `DefaultCodec` when one of these feature
 /// flags is toggled (`serde_bincode`, `serde_json`, `serde_cbor`, `serde_rmp`")
 #[cfg_attr(
@@ -197,27 +195,37 @@ pub struct Codec<R, W, C> {
     conn_type: PhantomData<C>,
 }
 
-/// WebSocket integration for async_tungstenite, tokio_tungstenite
-impl<S, E>
-    Codec<
-        StreamHalf<SplitStream<S>, CanSink>,
-        SinkHalf<SplitSink<S, WsMessage>, CanSink>,
-        ConnTypePayload,
-    >
-where
-    S: Stream<Item = Result<WsMessage, E>> + Sink<WsMessage> + Send + Sync + Unpin,
-    E: std::error::Error + 'static,
-{
-    /// Creates a `Codec` with a WebSocket connection.
-    ///
-    /// This works with the `WebSocketConn` types implemented in `async_tungstenite` and `tokio_tungstenite`
-    pub fn with_websocket(ws: WebSocketConn<S, CanSink>) -> Self {
-        let (writer, reader) = WebSocketConn::<S, CanSink>::split(ws);
+cfg_if! {
+    if #[cfg(any(feature = "ws_tokio", feature = "ws_async_std"))] {
+        use futures::stream::{SplitSink, SplitStream};
+        use futures::{Sink, Stream};
+        use tungstenite::Message;
 
-        Self {
-            reader,
-            writer,
-            conn_type: PhantomData,
+        use crate::transport::ws::{CanSink, SinkHalf, StreamHalf, WebSocketConn};
+
+        /// WebSocket integration for async_tungstenite, tokio_tungstenite
+        impl<S, E>
+            Codec<
+                StreamHalf<SplitStream<S>, CanSink>,
+                SinkHalf<SplitSink<S, Message>, CanSink>,
+                ConnTypePayload,
+            >
+        where
+            S: Stream<Item = Result<Message, E>> + Sink<Message> + Send + Sync + Unpin,
+            E: std::error::Error + 'static,
+        {
+            /// Creates a `Codec` with a WebSocket connection.
+            ///
+            /// This works with the `WebSocketConn` types implemented in `async_tungstenite` and `tokio_tungstenite`
+            pub fn with_websocket(ws: WebSocketConn<S, CanSink>) -> Self {
+                let (writer, reader) = WebSocketConn::<S, CanSink>::split(ws);
+
+                Self {
+                    reader,
+                    writer,
+                    conn_type: PhantomData,
+                }
+            }
         }
     }
 }
@@ -248,7 +256,12 @@ impl
 
 #[cfg(all(feature = "http_warp"))]
 // warp websocket
-impl<S, E> Codec<SplitStream<S>, SplitSink<S, warp::ws::Message>, ConnTypePayload>
+impl<S, E>
+    Codec<
+        StreamHalf<SplitStream<S>, CanSink>,
+        SinkHalf<SplitSink<S, warp::ws::Message>, CanSink>,
+        ConnTypePayload,
+    >
 where
     S: Stream<Item = Result<warp::ws::Message, E>> + Sink<warp::ws::Message>,
     E: std::error::Error,
@@ -258,6 +271,47 @@ where
     pub fn with_warp_websocket(ws: S) -> Self {
         use futures::StreamExt;
         let (writer, reader) = ws.split();
+        let writer = SinkHalf::<_, CanSink> {
+            inner: writer,
+            can_sink: PhantomData,
+        };
+        let reader = StreamHalf::<_, CanSink> {
+            inner: reader,
+            can_sink: PhantomData,
+        };
+
+        Self {
+            reader,
+            writer,
+            conn_type: PhantomData,
+        }
+    }
+}
+
+#[cfg(all(feature = "http_axum"))]
+impl<S, E>
+    Codec<
+        StreamHalf<SplitStream<S>, CanSink>,
+        SinkHalf<SplitSink<S, axum::ws::Message>, CanSink>,
+        ConnTypePayload,
+    >
+where
+    S: Stream<Item = Result<axum::ws::Message, E>> + Sink<axum::ws::Message>,
+    // E: std::error::Error
+{
+    /// Creates a codec with WebSocket wrapper type provided by `axum` HTTP framework
+    #[cfg_attr(feature = "docs", doc(cfg(feature = "http_axum")))]
+    pub fn with_axum_websocket(ws: S) -> Self {
+        use futures::StreamExt;
+        let (writer, reader) = ws.split();
+        let writer = SinkHalf::<_, CanSink> {
+            inner: writer,
+            can_sink: PhantomData,
+        };
+        let reader = StreamHalf::<_, CanSink> {
+            inner: reader,
+            can_sink: PhantomData,
+        };
 
         Self {
             reader,
@@ -269,19 +323,37 @@ where
 
 /// A codec that can read the header and body of a message
 #[async_trait]
-pub trait CodecRead: Unmarshal {
+pub trait CodecRead: Send + Unmarshal + EraseDeserializer {
     /// Reads the header of the message.
     async fn read_header<H>(&mut self) -> Option<Result<H, Error>>
     where
-        H: serde::de::DeserializeOwned;
+        H: serde::de::DeserializeOwned,
+    {
+        Some(
+            self.read_bytes()
+                .await?
+                .and_then(|payload| Self::unmarshal(&payload)),
+        )
+    }
 
     /// Reads the body of the message
-    async fn read_body(&mut self) -> Option<Result<RequestDeserializer, Error>>;
+    async fn read_body(&mut self) -> Option<Result<Box<InboundBody>, Error>> {
+        match self.read_bytes().await? {
+            Ok(payload) => {
+                let de = Self::from_bytes(payload);
+                Some(Ok(de))
+            }
+            Err(e) => return Some(Err(e)),
+        }
+    }
+
+    /// Reads the body as raw bytes
+    async fn read_bytes(&mut self) -> Option<Result<Vec<u8>, Error>>;
 }
 
 /// A codec that can write the header and body of a message
 #[async_trait]
-pub trait CodecWrite: Marshal {
+pub trait CodecWrite: Send + Marshal {
     /// Writes the header of the message
     async fn write_header<H>(&mut self, header: H) -> Result<(), Error>
     where
@@ -290,9 +362,12 @@ pub trait CodecWrite: Marshal {
     /// Writes the body of the message
     async fn write_body(
         &mut self,
-        id: &MessageId,
+        id: MessageId,
         body: &(dyn erased::Serialize + Send + Sync),
     ) -> Result<(), Error>;
+
+    /// Writes body as raw bytes
+    async fn write_body_bytes(&mut self, id: MessageId, bytes: &[u8]) -> Result<(), Error>;
 }
 
 cfg_if! {
