@@ -42,6 +42,7 @@
 //! - [Documentation](https://docs.rs/toy-rpc/0.7.0-alpha.0/toy_rpc/)
 //! - [GitHub repository](https://github.com/minghuaw/toy-rpc)
 //! - [crate.io](https://crates.io/crates/toy-rpc)
+//! - Minimum supported Rust version: 1.53 or later
 //!
 //! This crate uses `#![forbid(unsafe_code)]` to ensure no usage of `unsafe` in the crate.
 //!
@@ -53,9 +54,10 @@
 //!
 //! - `async_std_runtime`: supports usage with `async-std`
 //! - `tokio_runtime`: supports usage with `tokio`
-//! - `http_tide`: enables `tide` integration on the server side. This also enables `async_std_runtime`
-//! - `http_actix_web`: enables `actix-web` integration on the server side. This also enables `tokio_runtime`
-//! - `http_warp`: enables integration with `warp` on the server side. This also enables `tokio_runtime`
+//! - `http_tide`: enables `tide` integration on the server side. This also enables `async_std_runtime` and `ws_async_std`
+//! - `http_actix_web`: enables `actix-web` integration on the server side. This also enables `tokio_runtime` and `ws_tokio`
+//! - `http_warp`: enables integration with `warp` on the server side. This also enables `tokio_runtime` and `ws_tokio`
+//! - `http_axum`: enables integration with `axum` on the server side. This also enables `tokio_runtime` and `ws_tokio`
 //!
 //! Choice of RPC server or client (both can be enabled at the same time)
 //!
@@ -73,10 +75,17 @@
 //! - `serde_rmp`: the default codec will use `rmp-serde`
 //!     for serialization/deserialization
 //!
+//! WebSocket support (HTTP integration is implementd with WebSocket)
+//!
+//! - `ws_tokio`: enables WebSocket and HTTP integrations with `tokio`. 
+//! This must be enabled for client to use `dial_http(addr)` or `dial_websocket(addr)`.
+//! - `ws_async_std`: enables WebSocket and HTTP integrations with `async-std`. 
+//! This must be enabled for client to use `dial_http(addr)` or `dial_websocket(addr)`.
+//!
 //! TLS support
 //!
 //! - `tls`: enables TLS support
-//! 
+//!
 //! Other trivial feature flags are listed below, and they are likely of no actual usage for you.
 //! - `docs`
 //! - `std`: `serde/std`. There is no actual usage right now.
@@ -105,6 +114,8 @@ pub mod codec;
 pub mod error;
 pub mod macros;
 pub mod message;
+pub mod protocol;
+pub mod pubsub;
 pub mod service;
 pub mod transport;
 pub mod util;
@@ -114,6 +125,7 @@ pub mod util;
     feature = "http_actix_web",
     feature = "http_warp",
     feature = "http_tide",
+    feature = "http_axum",
     feature = "client"
 ))]
 pub const DEFAULT_RPC_PATH: &str = "_rpc_";
@@ -128,6 +140,9 @@ pub mod server;
 #[cfg(feature = "server")]
 pub use server::{builder::ServerBuilder, Server};
 
+/// Type alias for `std::result::Result<T, toy_rpc::error::Error>`
+pub type Result<T, E = error::Error> = std::result::Result<T, E>;
+
 pub use error::Error;
 
 // re-export
